#include "os.h"
#include "kheap.h"
#include "task.h"
#include "initrd.h"

extern ULONG file_data_start;
extern ULONG file_data_end;
extern ULONG prog_start;

extern page_directory_t* current_directory;
extern task_t* current_task;
extern task_t* ready_queue;
extern tss_entry_t tss_entry;
extern heap_t* kheap;

ULONG ramdisk_start;
UCHAR address_TEST[256];

void moo()
{
  while(TRUE)
  {
      settextcolor(2,0);
      printformat("MOO %d", getpid()); // <-- cow
      settextcolor(15,0);
  }
}

void baa()
{
  while(TRUE)
  {
      settextcolor(4,0);
      printformat("BAA %d", getpid()); // <- sheep
      settextcolor(15,0);
  }
}

void surprise(ULONG a, ULONG b) // <- function with 2 arguments
{
  asm volatile ("push %0" :/* no output registers */: "r" (b));
  asm volatile ("push %0" :/* no output registers */: "r" (a));
  while(TRUE)
  {
      asm volatile ("call _settextcolor");
      printformat("%d", getpid());
      settextcolor(15,0);
  }
}

void test()                    // <- program from outside "loaded" by incbin ...
{
  while(TRUE)
  {
      asm volatile("call *%0"::"r"(address_TEST));
      printformat("%d", getpid());
      settextcolor(15,0);
  }
}

int main()
{
    k_clear_screen();
    printformat("Welcome to PrettyOS ...\n");
    // GDT, IDT, ISRS, IRQ, timer, keyboard, paging, interrupts, multitasking
    gdt_install();
    idt_install();
    isrs_install();
    irq_install();
    initODA();
    timer_install();
    keyboard_install();
    sti();
    paging_install();

    // Initialise the kernel heap.
    kheap = create_heap(KHEAP_START, KHEAP_START+KHEAP_INITIAL_SIZE, KHEAP_MAX, 0, 0);

    tasking_install();

    settextcolor(2,0); printformat("VFS & RAM Disk install\n"); settextcolor(15,0);

    ramdisk_start = k_malloc(0x100000,0,0);
    k_memcpy((void*)ramdisk_start, &file_data_start, (ULONG)&file_data_end - (ULONG)&file_data_start); //process.asm

    fs_root = install_initrd(ramdisk_start);

    // list the content of files <- data from outside "loaded" by incbin ...
    ULONG i = 0;
    struct dirent* node = 0;
    while( (node = readdir_fs(fs_root, i)) != 0)
    {
        printformat("\nFound file %s\n",node->name);
        fs_node_t* fsnode = finddir_fs(fs_root, node->name);

        if((fsnode->flags & 0x7) == FS_DIRECTORY)
        {
            printformat("\t(directory)\n");
        }
        else
        {
            printformat("\t contents: \"");
            UCHAR buf[256];
            ULONG sz = read_fs(fsnode, 0, fsnode->length, buf);

            ULONG j;
            for(j=0; j<sz; ++j)
            {
                if( k_strcmp(node->name,"Test-Programm")==0 )
                {
                    printformat("%x ",buf[j]);
                    address_TEST[j] = buf[j];
                }
                else
                    printformat("%c",buf[j]);
            }
            printformat("\"\n");
        }
        ++i;
    }
    printformat("\naddress_TEST: %x",address_TEST);

    sleepSeconds(5);

    settextcolor(15,0);

    /* task_t* task2 = */ create_task(moo);
    /* task_t* task3 = */ create_task(baa);
    /* task_t* task4 = */ create_task2(surprise,4,2);
    /* task_t* task5 = */ create_task(test);

    k_clear_screen();
    log_task_list();
    sleepSeconds(5);

    pODA->ts_flag = 1; // enable task_switching

    k_clear_screen();
    while(TRUE)
    {
        settextcolor(15,0);
        printformat("%d", getpid());
    }
    return 0;
}
