#ifndef TASK_H
#define TASK_H

#include "os.h"
#include "paging.h"
#include "descriptor_tables.h"

#define KERNEL_STACK_SIZE 2048       // Use a 2kb kernel stack.

ULONG initial_esp;

// This structure defines a 'task' - a process.
typedef struct task
{
    int id;                           // Process ID.
    ULONG esp, ebp;                   // Stack and base pointers.
    ULONG eip;                        // Instruction pointer.
    page_directory_t* page_directory; // Page directory.
    ULONG kernel_stack;               // Kernel stack location.
    struct task* next;                // The next task in a linked list.
} task_t;

// Initialises the tasking system.
void tasking_install();

// Called by the timer hook, this changes the running process.
ULONG task_switch1(ULONG esp);

// Forks the current process, spawning a new one with a different memory space.
int fork();

// Returns the pid of the current process.
int getpid();
task_t* create_task(void* entry);
task_t* create_task2(void* entry, ULONG arg1, ULONG arg2);
task_t* create_task0(void* entry);
void task_log(task_t* t);
void TSS_log(tss_entry_t* tss);
void log_task_list();

#endif
